import 'package:flutter/gestures.dart';
import 'package:flutter/widgets.dart';

class CLXIntervalBtn extends StatelessWidget with WidgetInterval {
  CLXIntervalBtn(
      {super.key,
      required this.child,
      interval = const Duration(seconds: 1),
      this.disabled = false,
      this.onTap});

  final Widget? child;
  final bool disabled;
  final Function? onTap;

  @override
  Widget build(BuildContext context) {
    return GestureDetector(
      key: key,
      onTap: _onTap,
      child: child,
    );
  }

  void _onTap() {
    if (disabled) {
      return;
    }
    if (lastTap == null ||
        lastTap?.add(interval).compareTo(DateTime.now()) == -1) {
      lastTap = DateTime.now();
      onTap?.call();
    }
  }
}

class CLXTapGestureRecognizer extends TapGestureRecognizer
    with TapGestureRecognizerInterval {
  @protected
  @override
  void handleTapUp(
      {required PointerDownEvent down, required PointerUpEvent up}) {
    final TapUpDetails details = TapUpDetails(
      kind: up.kind,
      globalPosition: up.position,
      localPosition: up.localPosition,
    );

    final canTouch = lastTap == null ||
        lastTap?.add(interval).compareTo(DateTime.now()) == -1;

    if (!canTouch) {
      return;
    }
    lastTap = DateTime.now();

    switch (down.buttons) {
      case kPrimaryButton:
        if (onTapUp != null) {
          invokeCallback<void>('onTapUp', () => onTapUp!(details));
        }
        if (onTap != null) {
          invokeCallback<void>('onTap', onTap!);
        }
      case kSecondaryButton:
        if (onSecondaryTapUp != null) {
          invokeCallback<void>(
              'onSecondaryTapUp', () => onSecondaryTapUp!(details));
        }
        if (onSecondaryTap != null) {
          invokeCallback<void>('onSecondaryTap', () => onSecondaryTap!());
        }
      case kTertiaryButton:
        if (onTertiaryTapUp != null) {
          invokeCallback<void>(
              'onTertiaryTapUp', () => onTertiaryTapUp!(details));
        }
      default:
    }
  }
}

mixin WidgetInterval on Widget {
  Duration get interval => _interval;

  Duration _interval = Duration.zero;

  DateTime? lastTap;

  set interval(Duration duration) {
    _interval = duration;
  }
}

mixin TapGestureRecognizerInterval on TapGestureRecognizer {
  Duration get interval => _interval;

  Duration _interval = const Duration(seconds: 1);

  DateTime? lastTap;

  set interval(Duration duration) {
    _interval = duration;
  }
}
